//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//

#ifndef VKDetectorConstruction_H
#define VKDetectorConstruction_H 1

#include "G4Material.hh"
#include "G4VisAttributes.hh"
#include "G4RotationMatrix.hh"
#include "G4VUserDetectorConstruction.hh"
#include "G4Cache.hh"

#include "VKDetectorMessenger.hh"

// forward class declarations
class G4LogicalVolume;
class G4VPhysicalVolume;
class G4Box;
class G4Tubs;
class VKMagnet;
class VKBetaDetector;
class VKBetaDetectorV2;
class VKMaterials;
class VKCatcherFoil;
class VKShield;
class VKLeadBlock;
class VKBlinkers;
class G4FieldManager;
class G4MagneticField;
class VKSiPMSensitiveDetector;
class VKVirtualDetector;
class VKScintillatorSensitiveDetector;


class VKDetectorConstruction : public G4VUserDetectorConstruction
{
  public:

    VKDetectorConstruction();
    virtual ~VKDetectorConstruction();

    virtual G4VPhysicalVolume* Construct();
    virtual void ConstructSDandField();

    //
    // Methods to modify the setup:
    //
        
    /*
     * Add or remove the magnet from setup 
     * @param b If true, magnet is created. If false, magnet is removed/not created
     */
    void SetMagnetOn(G4bool b);

    /*
     * Add or remove the beta detector (v1.0) from setup 
     * @param b If true, detector is created. If false, detector is removed/not created
     */
    void SetBetaDetectorOn(G4bool b, bool defaultPos = true);

    /*
     * Add or remove the beta detector (v2.0) from setup 
     * @param b If true, detector is created. If false, detector is removed/not created
     * @param defaultPos Return detector to default position and orientation
     */
    void SetBetaDetectorV2On(G4bool b, bool defaultPos = true);

    /*
     * Add or remove the muon telescope from setup 
     * @param b If true, telescope is created. If false, telescope is removed/not created
     */
    void SetMuonTelescopeOn(G4bool b);

    /*
     * Add or remove the catcher foil from the setup 
     * @param b If true, catcher foil is created. If false, catcher foil is removed/not created
     */
    void SetCatcherFoilOn(G4bool b);

    /*
     * Add or remove the shield from the setup 
     * @param b If true, shield is created. If false, shield is removed/not created
     */
    void SetShieldOn(G4bool b);

    /*
     * Add or remove lead block from the setup.
     * @param b If true, block is created. If false, block is removed/not created.
     *
     * The block has dimensions (10cm, 5cm, 10cm). The center of the block is at (0.0cm, 7.5cm, 32cm).
     */
    void SetLeadBlockOn(G4bool b);

    /*
     * Add or remove plastic blinkers from the setup.
     * @param b If true, blinkers are created. If false, blinkers are removed/not created.
     */
    void SetBlinkersOn(G4bool b);

    /*
     * Add or remove virtual detectors from the setup 
     * @param b If true, virtual detectors are created. If false, they are removed/not created
     */
    void SetVirtualDetectorsOn(G4bool b);

    /*
     * Enable/disable the magnetic field 
     * @param b If true, field is enabled. If false, field is disabled
     */
    void SetBFieldOn(G4bool b);

    /*
     * Set the maximum strength of the magnetic field 
     * @param b Maximum field strength
     */
    void SetMaxBField(G4double b);

    /*
     * Set the reflectivity of the coating surrounding the plastic scintillator
     * @param refl Reflectivity (between 0 and 1)
     */
    void SetCoatingReflectivity(G4double refl);

    /*
     * Check for overlaps every time a physical volume is (re)created
     * @param b If true, check is perfored. If false, check is not performed
     */
    inline void SetCheckOverlaps(G4bool b) { fCheckOverlaps=b; }

    /*
     * Set position of catcher foil
     */    
    void SetCatcherFoilPosition(const G4ThreeVector& pos);

    /*
     * Set thickness of catcher foil
     */    
    void SetCatcherFoilThickness(G4double thickness);

    /*
     * Set material of catcher foil
     */    
    void SetCatcherFoilMaterial(G4String name);

    /*
     * Set position of beta detector
     */    
    void SetBetaDetectorPosition(const G4ThreeVector& pos);

    /*
     * Set position of beta detector V2.0
     */    
    void SetBetaDetectorV2Position(const G4ThreeVector& pos);

    //
    // Methods to obtain information about the setup:
    //
        
    /*
     * Should GEANT4 check for overlaps?
     */    
    inline G4bool GetCheckOverlaps() const { return fCheckOverlaps; }    
    
    /*
     * Is the magnet included in the setup?
     */    
    inline G4bool GetMagnetOn() const { return fMagnetOn; }
    
    /*
     * Is the beta detector v1.0 included in the setup?
     */    
    inline G4bool GetBetaDetectorOn() const { return fBetaDetectorOn; }

    /*
     * Is the beta detector v2.0 included in the setup?
     */    
    inline G4bool GetBetaDetectorV2On() const { return fBetaDetectorV2On; }

    /*
     * Is the catcher foil included in the setup?
     */    
    inline G4bool GetCatcherFoilOn() const { return fCatcherFoilOn; }

    /*
     * Is the shield included in the setup?
     */    
    inline G4bool GetShieldOn() const { return fShieldOn; }

    /*
     * Are the virtual detectors in the setup?
     */    
    inline G4bool GetVirtualDetectorsOn() const { return fVirtualDetectorsOn; }

    /*
     * Has the magnetic field been enable?
     */    
    inline G4bool GetBFieldOn() const { return fBFieldOn; }

    /*
     * Get position of beta detector
     */    
    inline const G4ThreeVector& GetBetaDetectorPosition() const { return fBetaDetectorPosition; }

    /*
     * Get position of catcher foil
     */    
    inline const G4ThreeVector& GetCatcherFoilPosition() const { return fCatcherFoilPosition; }

    /*
     * Get thickness of catcher foil
     */    
    inline G4double GetCatcherFoilThickness() const { return fCatcherFoilThickness; }

    /*
     * Get material of catcher foil
     */    
    inline G4String GetCatcherFoilMaterial() const { return fCatcherFoilMaterial; }

    /*
     * Get reflectivity of coating surrounding plastic scintillator
     */    
    inline G4double GetCoatingReflectivity() const { return fRefl; }

    /*
     * Get bore diameter
     */    
    inline G4double GetBoreDiameter() const { return fBoreDiameter; }

    /*
     * Get diameter of shield (at its widest)
     */    
    inline G4double GetShieldDiameter() const { return fShieldDiameter; }

    /*
     * Get diameter of entrance hole along beam axis
     */    
    inline G4double GetIronEndcapHoleDiameter() const { return fIronEndcapHoleDiameter; }

    /*
     * Get total length of spectrometer (outer wall to outer wall)
     */    
    inline G4double GetSpectrometerLength() const { return fSpectrometerLength; }

    /*
     * Get thickness of iron endcap.
     */    
    inline G4double GetIronEndcapThickness() const { return fIronEndcapThickness; }

    /*
     * Get length of iron cone.
     */    
    inline G4double GetIronConeLength() const { return fIronConeLength; }

    /*
     * Get pointer to focal-plane detector
     */
    inline const VKVirtualDetector* GetFocalPlane() const { return fFocalPlane; }

    /*
     * Get pointer to midway-plane detector
     */
    inline const VKVirtualDetector* GetMidwayPlane() const { return fMidwayPlane; }

    /*
     * Get pointer to beta detector v1.0
     */
    inline const VKBetaDetector* GetBetaDetector() const { return fBetaDetector; }

    /*
     * Get pointer to beta detector v2.0
     */
    inline const VKBetaDetectorV2* GetBetaDetectorV2() const { return fBetaDetectorV2; }


  private:

    G4VPhysicalVolume* ConstructDetector();
    void SetDefaults();

    VKDetectorMessenger* fDetectorMessenger;

    // magnetic field
    G4FieldManager* fBFieldMgr;
    G4Cache<G4MagneticField*> fBField;  // pointer to the thread-local fields
    bool fBFieldOn;
    G4double fMaxBField;    
    
    // world
    G4Box *fWorld_sol;
    G4LogicalVolume *fWorld_log;
    G4VPhysicalVolume *fWorld_phys;

    // Elements, materials, and material properties
    VKMaterials *fMaterials;
    G4Material *fLabVacuum;
    
    // Geometry
    bool fCheckOverlaps;
    
    // Magnet
    G4double fSpectrometerLength;
    G4double fBoreDiameter;
    G4double fIronEndcapHoleDiameter;
    G4double fIronEndcapThickness;
    G4double fIronConeLength;
    bool fMagnetOn;
    VKMagnet *fMagnet;
    
    G4double fRefl;
    
    G4ThreeVector fBetaDetectorDefaultPosition;
    G4RotationMatrix fBetaDetectorDefaultRotation; 

    // Beta detector
    G4ThreeVector fBetaDetectorPosition;
    G4RotationMatrix fBetaDetectorRotation;
    bool fBetaDetectorOn;
    VKBetaDetector *fBetaDetector;    

    // Beta detector V2
    G4ThreeVector fBetaDetectorV2Position;
    G4RotationMatrix fBetaDetectorV2Rotation;
    bool fBetaDetectorV2On;
    VKBetaDetectorV2 *fBetaDetectorV2;    

    // Catcher foil
    G4String fCatcherFoilMaterial;
    G4ThreeVector fCatcherFoilPosition;
    G4double fCatcherFoilThickness;
    bool fCatcherFoilOn;
    VKCatcherFoil *fCatcherFoil;
    
    // Shield 
    G4double fShieldDiameter;
    bool fShieldOn;
    VKShield *fShield;

    // Lead block 
    bool fLeadBlockOn;
    VKLeadBlock *fLeadBlock;

    // Blinkers
    bool fBlinkersOn;
    VKBlinkers *fBlinkers;
    
    // Virtual detectors
    bool fVirtualDetectorsOn;
    VKVirtualDetector *fFocalPlane, *fMidwayPlane;

    // Sensitive Detectors (BetaDetector)
    G4bool fSetSDs;
    G4Cache<VKScintillatorSensitiveDetector*> fBetaDetSignal_SD, fBetaDetVeto_SD;
    G4Cache<VKSiPMSensitiveDetector*> fSiPM_SD;

    // Sensitive Detectors (BetaDetectorV2)
    G4bool fSetV2SDs;
    G4Cache<VKScintillatorSensitiveDetector*> fBetaDetV2Front_SD, fBetaDetV2Signal_SD, fBetaDetV2Veto_SD;
    G4Cache<VKSiPMSensitiveDetector*> fSiPMV2_SD;
};

#endif
