//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
#include "globals.hh"

#include "VKCatcherFoil.hh"
#include "VKUtil.hh"

#include "G4LogicalSkinSurface.hh"
#include "G4LogicalBorderSurface.hh"
#include "G4SystemOfUnits.hh"
#include "G4Cons.hh"
#include "TMath.h"


VKCatcherFoil::VKCatcherFoil(G4RotationMatrix *pRot,
                             const G4ThreeVector &tlate,
                             G4LogicalVolume *pMother_log,
                             G4bool pMany,
                             G4int pCopyNo,
                             const VKDetectorConstruction* pDetConstr)

 // Pass info to the G4PVPlacement constructor
 : G4PVPlacement(pRot, 
                 tlate,
                 CreateTemporaryLogicalVolume(), // <-- see VKUtil.cpp
                 "catcherFoil", 
                 pMother_log, 
                 pMany, 
                 pCopyNo),
   fDetConstr(pDetConstr),              
   fMother_log(pMother_log)
{

    // default dimensions    
    SetDefaults();

    // List of materials
    fMaterials = VKMaterials::GetInstance();

    // Get materials
    G4Material* labVacuum = fMaterials -> GetMaterial("LABVACUUM");
    G4Material* carbon    = fMaterials -> GetMaterial(fFoilMaterial);
    G4Material* alu       = fMaterials -> GetMaterial("ALUMINIUM");
    assert(carbon != nullptr); // check that material exists
    assert(alu != nullptr);
    assert(labVacuum != nullptr);

    // Initialize variables
    G4double startAngle, spanningAngle;


    // -------------------------------------------- //
    //             (Imaginary) container            //
    // -------------------------------------------- //
    
    // *** All other volumes should fit inside this volume !

    // solid
    fContainer_sol = new G4Tubs("catcherFoil_sol", 0., fContainerDiameter/2, 
                              fContainerLength/2, 0.*deg, 360.*deg);

    // logical
    fContainer_log = new G4LogicalVolume(fContainer_sol, labVacuum, "catcherFoil_log");
    
        
    // ------------------------ //
    //           Foil           //
    // ------------------------ //

    // solid 
    fFoil_sol = new G4Tubs("foil_sol", 0., fFoilDiameter/2, fFoilThickness/2, 
                           startAngle=0.*deg, spanningAngle=360.*deg);
    // logical 
    fFoil_log = new G4LogicalVolume(fFoil_sol, carbon, "foil_log");  
  
    // physical 
    new G4PVPlacement(0, G4ThreeVector(), fFoil_log, "foil", fContainer_log, false, 
                      0, fCheckOverlaps);

    // ------------------------ //
    //          Frame           //
    // ------------------------ //
    
    // solid
    fFrame_sol = new G4Tubs("frame_sol", fFoilDiameter/2, fFrameOuterDiameter/2,         
                            fFrameThickness/2, startAngle=0.*deg, spanningAngle=360.*deg);
    
    // logical 
    fFrame_log = new G4LogicalVolume(fFrame_sol, alu, "frame_log");  
  
    // physical 
    new G4PVPlacement(0, G4ThreeVector(0,0,fFrameThickness/2), fFrame_log, "frame", fContainer_log, false, 
                      0, fCheckOverlaps);


    // visualization attributes
    VisAttributes();    


    // Set the logical volume
    this -> SetLogicalVolume(fContainer_log);
    
    // Move the detector so that 'tlate' coincides with the
    // center of the foil
    this -> SetTranslation(tlate);
    
    // Check for overlaps
    if (fCheckOverlaps) this -> CheckOverlaps();
}


void VKCatcherFoil::SetDefaults(){

    fCheckOverlaps = fDetConstr -> GetCheckOverlaps();

    // Dimensions
    fFoilMaterial       = fDetConstr->GetCatcherFoilMaterial();
    fFoilDiameter       = 14.*mm;
    fFoilThickness      = fDetConstr->GetCatcherFoilThickness();
    fFrameOuterDiameter = 22*mm;
    fFrameThickness     = 0.5*mm;

    // deduce container size
    fContainerDiameter = fFrameOuterDiameter;
    fContainerLength   = TMath::Max(fFoilThickness, fFrameThickness);
}


void VKCatcherFoil::VisAttributes() {

    // foil
    G4VisAttributes* foil_va = new G4VisAttributes(G4Colour(1.0,1.0,1.0)); // white
    foil_va   -> SetVisibility(true);
    foil_va   -> SetForceSolid(true);
    fFoil_log -> SetVisAttributes(foil_va);

    // frame
    G4VisAttributes* frame_va = new G4VisAttributes(G4Colour(0.5,0.5,0.5)); // gray
    frame_va   -> SetVisibility(true);
    frame_va   -> SetForceSolid(true);
    fFrame_log -> SetVisAttributes(frame_va);
}

